;**************************************************************************
;* FileName	: 24CXX.INC
;* Date 	: 09.10.2002
;* File version : v1.01
;*
;* Author	: MIKO, Imre
;* Company	:
;* Homepage	: http://majki.fw.hu
;* E-mail	: majki@fw.hu
;*
;*-------------------------------------------------------------------------
;*
;* Description	: Functions for 24cxx EEPROMs
;*		  ==================================
;*		  EE24_writebyte
;*		  EE24_writepage
;*		  EE24_readbyte
;*		  EE24_readpage
;*
;*
;*-------------------------------------------------------------------------
;* Needed defines and includes in main-modul:
;*
;* xtal_freq	EQU	4000000	  	; crystal frequency
;* core_freq	EQU	xtal_freq/4	; core operating frequency
;*
;* #define		I2C_SCL		PORTC,3		; out
;* #define		I2C_SDA		PORTC,4		; in/out
;* #define		EE24_8BIT_ADDRESS		; It must be defined only 
;*							; for 24LC00 ... 24LC16 devices
;*
;*	include "..\..\modules\i2c_hw.mac"
;*	include "..\..\modules\i2c_hw.inc"
;*
;*-------------------------------------------------------------------------
;* Other possible defines in main-modul:
;*
;*  <none>
;*
;**************************************************************************
;* Revision History:
;*
;* 1.00	  23/02/2001 	Initial Release
;* 1.01   09/10/2002    - reformatting source
;*			- support for small devices
;*
;**************************************************************************

	ifdef	INC_EE24

;***** VARIABLES **********************************************************

	cblock
		EE24_temp 
	 	EE24_addr16:0
		EE24_addr:0, EE24_addr_h, EE24_addr_l
	endc



;***** EE24_writebyte *****************************************************
;* Write a byte into EEPROM
;* Input : w <- data			; 
;*	   EE24_addr <- target address	; (16 bit register pair)
;* Output: -				;
;* Clock : ?	
;**************************************************************************
EE24_writebyte	movwf	EE24_temp
		I2C_STARTCOND

EE24_wb_ackpoll	
		ifndef EE24_8BIT_ADDRESS
		  movlw	EE24_WRITE
		else
		  clrc
		  rlf	EE24_addr_h,w
		  iorlw	EE24_WRITE
		endif
		
		I2C_SEND		; select slave
		jnz	EE24_wb_ackok

		I2C_RESTARTCOND
		goto	EE24_wb_ackpoll

EE24_wb_ackok	
		ifndef	EE24_8BIT_ADDRESS
		  movf	EE24_addr_h,w
		  I2C_SEND		; Mem. addr. high
		endif

		movf	EE24_addr_l,w	
		I2C_SEND		; Mem. addr. low

		movf	EE24_temp,w			
		I2C_SEND		; data
		I2C_STOPCOND	
		return



;***** EE24_writepage *****************************************************
;* Write n bytes into EEPROM
;* Input : FSR <- startaddress of data	;
;*	   IRP <- indirect bank select	; for startaddress of data
;*	   w   <- length of page	;  
;*	   EE24_addr <- target address	; (16 bit register pair)
;* Output: -				;
;* Clock : ?		
;**************************************************************************
EE24_writepage	movwf	EE24_temp
		I2C_STARTCOND

EE24_wp_ackpoll	
		ifndef EE24_8BIT_ADDRESS
		  movlw	EE24_WRITE
		else
		  clrc
		  rlf	EE24_addr_h,w
		  iorlw	EE24_WRITE
		endif

		I2C_SEND		; select slave
		jnz	EE24_wp_ackok	

		I2C_RESTARTCOND
		goto	EE24_wp_ackpoll

EE24_wp_ackok	
		ifndef	EE24_8BIT_ADDRESS
		  movf	EE24_addr_h,w
		  I2C_SEND		; Mem. addr. high
		endif

		movf	EE24_addr_l,w	
		I2C_SEND		; Mem. addr. low

EE24_wp_wrnext	movf	INDF,w
		incf	FSR,f			
		I2C_SEND		; data
		decfsz	EE24_temp,f
		goto	EE24_wp_wrnext

		I2C_STOPCOND	
		return


;***** EE24_readbyte ******************************************************
;* Read a byte from EEPROM
;* Input : EE24_addr <- source address	; (16 bit register pair)
;* Output: data -> w			;
;* Clock : ?
;**************************************************************************
EE24_readbyte	I2C_STARTCOND

EE24_rb_ackpoll	
		ifndef EE24_8BIT_ADDRESS
		  movlw	EE24_WRITE
		else
		  clrc
		  rlf	EE24_addr_h,w
		  iorlw	EE24_WRITE
		endif

		I2C_SEND		; select slave
		jnz	EE24_rb_ackok

		I2C_RESTARTCOND
		goto	EE24_rb_ackpoll

EE24_rb_ackok	
		ifndef	EE24_8BIT_ADDRESS
		  movf	EE24_addr_h,w
		  I2C_SEND		; Mem. addr. high
		endif

		movf	EE24_addr_l,w	
		I2C_SEND		; Mem. addr. low

		I2C_RESTARTCOND
		
		ifndef EE24_8BIT_ADDRESS
		  movlw	EE24_READ
		else
		  clrc
		  rlf	EE24_addr_h,w
		  iorlw	EE24_READ
		endif

		I2C_SEND		; select slave

		I2C_RECEIVE
		movwf	EE24_temp

		clrc
		I2C_ACK			; no ACK

		I2C_STOPCOND
		movf	EE24_temp,w
		return



;***** EE24_readpage ******************************************************
;* Read n byte from EEPROM
;* Input : FSR <- target address	;
;*	   IRP <- indirect bank select	; for target address
;*	   w   <- length of page	;  
;*	   EE24_addr <- source address	; (16 bit register pair)
;* Output: data -> target area 				;
;* Clock : ?		
;**************************************************************************
EE24_readpage	movwf	EE24_temp
		I2C_STARTCOND
EE24_rp_ackpoll	
		ifndef EE24_8BIT_ADDRESS
		  movlw	EE24_WRITE
		else
		  clrc
		  rlf	EE24_addr_h,w
		  iorlw	EE24_WRITE
		endif

		I2C_SEND		; select slave
		jnz	EE24_rp_ackok

		I2C_RESTARTCOND
		goto	EE24_rp_ackpoll
	
EE24_rp_ackok	
		ifndef	EE24_8BIT_ADDRESS
		  movf	EE24_addr_h,w
		  I2C_SEND		; Mem. addr. high
		endif

		movf	EE24_addr_l,w	
		I2C_SEND		; Mem. addr. low

		I2C_RESTARTCOND

		ifndef EE24_8BIT_ADDRESS
		  movlw	EE24_READ
		else
		  clrc
		  rlf	EE24_addr_h,w
		  iorlw	EE24_READ
		endif

		I2C_SEND		; select slave
		goto	EE24_rp_rec

EE24_rp_next	setc
		I2C_ACK			; ACK

EE24_rp_rec	I2C_RECEIVE
		movwf	INDF
		incf	FSR,f
		decfsz	EE24_temp,f
		goto	EE24_rp_next

EE24_rp_end	clrc
		I2C_ACK			; no ACK
		I2C_STOPCOND		
		return



;**************************************************************************
	endif


