;**************************************************************************
;* FileName	: BASE.INC
;* Date 	: 18.06.2002
;* File version : v1.04
								nolist
;* Author	: MIKO, Imre
;* Company	:
;* Homepage	: http://majki.fw.hu
;* E-mail	: majki@fw.hu
;*
;*-------------------------------------------------------------------------
;*
;* Description	: Base
;*		  ==================================
;*		  MULTIPAGE
;*		  EXPAND_MACROS
;*		  XCHG
;*		  MEM_CLEAR
;*
;*-------------------------------------------------------------------------
;* Needed defines and includes in main-modul:
;*
;*  <none>
;*
;*-------------------------------------------------------------------------
;* Other possible defines in main-modul:
;*
;*  <none>
;*
;**************************************************************************
;* Revision History:
;*
;* 1.00	  10/02/2001 	Initial Release
;* 1.01	  23/08/2001 	Minor changes (labels)
;* 1.02	  03/11/2001	Add: INCLUDEPATH
;* 1.03   04/04/2002	New Macros: clrw, XCHGW, XCHG
;* 1.04   18/06/2002	Support for the multipage program memory
;*
;**************************************************************************


;***** DEFINES ************************************************************

#define		MHz		* 1000000
#define		kHz		* 1000

#define		INCLUDEPATH	d:\work\pic\modules


; ----- macro expansion modes ---------------------------------------------

#define		EXP_NONE	0
#define		EXP_COND	1
#define		EXP_ALL		2

#define		EXP		expand
#define		NOEXP		noexpand

; ----- instruction redefines ---------------------------------------------
#define		bank0		bcf	STATUS, RP0
#define		bank1		bsf	STATUS, RP0
#define		bankLO		bcf	STATUS, RP1
#define		bankHI		bsf	STATUS, RP1
#define		ibankLO		bcf	STATUS, IRP
#define		ibankHI		bsf	STATUS, IRP

#define		nop2		goto	$+1
#define         skipz		btfss	STATUS, Z
#define         skipnz		btfsc	STATUS, Z
#define         skipc		btfss	STATUS, C
#define         skipnc		btfsc	STATUS, C
#define         skipdc		btfss	STATUS, DC
#define         skipndc		btfsc	STATUS, DC
#define		jc		bc
#define		jnc		bnc
#define		jz		bz
#define		jnz		bnz
#define		testf		tstf
#define		testw		xorlw	0

#define		decw		addlw	-1
#define		incw		addlw	1
#define		clrw		movlw	0

#define		clrz		bcf	STATUS, Z
#define		clrc		bcf	STATUS, C
#define		setz		bsf	STATUS, Z
#define		setc		bsf	STATUS, C

; ----- multipage support -------------------------------------------------
#define		fcall		call	; far call
#define		fgoto		goto	; far goto
#define		NUMBER_OF_PAGE	1
		variable BANKSIZE_BITS = 11	; default for 14 bit devices



;***** MULTIPAGE **********************************************************
;* support multipage program memory   
;* Param : -            		; 
;* Mem   : -
;* Clock : -
;**************************************************************************
MULTIPAGE	macro numofpage, banksize
									NOEXP
		local size
#undefine	NUMBER_OF_PAGE
#define		NUMBER_OF_PAGE	numofpage

BANKSIZE_BITS=0
size=banksize>>1
	while ((size)!=0)
size>>=1
BANKSIZE_BITS++
	endw

#undefine	fcall
#define		fcall		lcall	; far call 
#undefine	fgoto
#define		fgoto		lgoto	; far goto
									EXP
		endm



;***** EXPAND_MACROS ******************************************************
;* set macro expansion mode   
;* Param : mode            		; EXP_NONE, EXP_COND, EXP_ALL
;* Mem   : -
;* Clock : -
;**************************************************************************
EXPAND_MACROS	macro	mode
									NOEXP
#undefine	EXP
#undefine	NOEXP	

	if (mode==EXP_NONE)
#define		EXP		noexpand
#define		NOEXP		noexpand
	endif
								
	if (mode==EXP_COND)
#define		EXP		expand
#define		NOEXP		noexpand
	endif

	if (mode==EXP_ALL)
#define		EXP		expand
#define		NOEXP		expand
	endif
									EXP
		endm



;***** XCHGW **************************************************************
;* eXchange file registers (W <-> file1)
;* Param : file1            		; 
;* Mem   : 3
;* Clock : 3
;**************************************************************************
XCHGW		macro file1
		;==========================================================
			xorwf	file1, f
			xorwf	file1, w
			xorwf	file1, f


		endm


;***** XCHG ***************************************************************
;* eXchange file registers (file1 <-> file2)
;* Param : file1            		; 
;*         file2			; 
;* Mem   : 4
;* Clock : 4
;**************************************************************************
XCHG		macro file1, file2
		;==========================================================
			movf	file1,w                 
			subwf	file2,w
			addwf	file1,f
			subwf	file2,f


		endm



;***** MEM_CLEAR macro ****************************************************
;* Clear file-memory array 
;* Param : addr             		; start address of mem.
;*         count			; number of bytes must be clear
;* Mem   : 8-9
;* Clock : 6*count + 4   
;**************************************************************************
MEM_CLEAR	macro	addr, count
		;==========================================================
									NOEXP
			local MEM_l1
			if addr > 0FFh
									EXP								   EXP
			  ibankHI
									NOEXP
			endif
									EXP
			movlw	addr & 0FFh	; clear header
			movwf	FSR
			movlw	count
MEM_l1
			clrf	INDF
			incf	FSR,f
			decw
			jnz	MEM_l1
									NOEXP
			if addr > 0FFh
									EXP
			  ibankLO
									NOEXP
		  	endif
									EXP


		endm



;**************************************************************************
	list

