;**************************************************************************
;* FileName	: MATH.INC
;* Date		: 11.10.2002
;* File version	: v1.02
;*
;* Author	: MIKO, Imre
;* Company	:
;* Homepage	: http://majki.fw.hu
;* E-mail	: majki@fw.hu
;*
;*-------------------------------------------------------------------------
;*
;* Description	: math functions
;*		  ==================================
;*		  mul0808	mul1616
;*		  div0808	div1616
;*
;*		  bin2bcd8	bin2bcd16
;*		  bin2bcd8i	bin2bcd16i
;*
;*		  bcd_inc8
;*		  bcd_inc8i
;*
;*		  bcd_dec8
;*		  bcd_dec8i
;*
;*-------------------------------------------------------------------------
;* Needed defines and includes in main-modul:
;* 
;*  <none>
;*
;*-------------------------------------------------------------------------
;* Other possible defines in main-modul:
;*
;* #define MATH_OPTSPEED		; Default optimalisation of the math
;*					; routines is "size". If MATH_OPTSPEED
;*					; defined optimalisation of the math
;*					; routines is "speed".
;*
;**************************************************************************
;* Revision History:
;*
;* 1.00	  02/01/2002 	Initial Release
;* 1.01   10/10/2002	reformatting source
;* 1.02   11/10/2002	new functions: bcd_dec8, bcd_dec8i, bcd_inc8, bcd_inc8i
;*
;**************************************************************************

;***** VARIABLES **********************************************************

; ----- general variables -------------------------------------------------
	if (MATH_AARGSIZE != 0)
		cblock
		  LOOPCOUNT
		endc
	endif

; ----- arguments ---------------------------------------------------------
	MATH_PUTARG AARG, AARGB, MATH_AARGSIZE
	MATH_PUTARG BARG, BARGB, MATH_BARGSIZE
	MATH_PUTARG REM,  REMB,  MATH_REMSIZE
	MATH_PUTARG TEMP, TEMPB, MATH_TEMPSIZE


;***** MUL & DIV MACROS ***************************************************

	ifdef INC_MATH0808
	  #include INCLUDEPATH\math0808.mac
	endif

	ifdef INC_MATH1616
	  #include INCLUDEPATH\math1616.mac
	endif

;***** FUNCTIONS **********************************************************


;***** mul0808 ************************************************************
;* 8x8 Bit Unsigned Fixed Point Multiply 8x8 -> 16
;* Input : AARG8 <- 8 bit unsigned fixed point multiplicand
;*         BARG8 <- 8 bit unsigned fixed point multiplier
;* Output: 16 bit unsigned fixed point product -> AARG16
;* Clock : 56-73  (24-38)
;**************************************************************************
	ifdef	INC_MUL0808
mul0808		clrf	AARGB1		; clear partial product

		ifdef	MATH_OPTSPEED
		  UMUL0808
		else
		  UMUL0808L
		endif

		retlw	0
	endif



;***** div0808 ************************************************************
;* 8/8 Bit Unsigned Fixed Point Divide 8/8 -> 08.08
;* Input : AARG8 <- 8 bit unsigned fixed point dividend
;*         BARG8 <- 8 bit unsigned fixed point divisor
;* Output: 8 bit unsigned fixed point quotient  -> AARG8
;*	   8 bit unsigned fixed point remainder -> REM8
;* Clock : 92-100  (67-75)
;**************************************************************************
	ifdef	INC_DIV0808
div0808		clrf	REMB0

		ifdef	MATH_OPTSPEED
		  UDIV0808
		else
		  UDIV0808L
		endif

		retlw	0
	endif



;***** mul1616 ************************************************************
;* 16x16 Bit Unsigned Fixed Point Multiply 16x16 -> 32
;* Input : AARG16 <- 16 bit unsigned fixed point multiplicand
;*         BARG16 <- 16 bit unsigned fixed point multiplier
;* Output: 32 bit unsigned fixed point product -> AARG32
;* Clock : 105-252 (41-184)
;**************************************************************************
	ifdef	INC_MUL1616
mul1616		clrf	AARGB2		; clear partial product
		clrf	AARGB3
		movf	AARGB0,w
		movwf	TEMPB0
		movf	AARGB1,w
		movwf	TEMPB1

		ifdef	MATH_OPTSPEED
		  UMUL1616
		else
		  UMUL1616L
		endif

		retlw	0
	endif



;***** div1616 ************************************************************
;* 16/16 Bit Unsigned Fixed Point Divide 16/16 -> 16.16
;* Input : AARG16 <- 16 bit unsigned fixed point dividend
;*         BARG16 <- 16 bit unsigned fixed point divisor
;* Output: 16 bit unsigned fixed point quotient  -> AARG16
;*	   16 bit unsigned fixed point remainder -> REM16
;* Clock : 277-373 (228-324)
;**************************************************************************
	ifdef	INC_DIV1616
div1616		clrf	REMB0
		clrf	REMB1

		ifdef	MATH_OPTSPEED
		  UDIV1616
		else
		  UDIV1616L
		endif

		retlw	0
	endif



;***** bin2bcd8i **********************************************************
;* Convert a binary number to BCD
;* Input : w <-address of 8 bit binary number (Bank must be selected!)
;* Output: AARGB0:AARGB1		; (AARG16) converted number (3 digit right justified)
;* Clock : ?	
;**************************************************************************
	ifdef	INC_BIN2BCD8I
bin2bcd8i	movwf	FSR
		movf	INDF,w
	endif
					; cont. with bin2bcd8


;***** bin2bcd8 ***********************************************************
;* Convert a binary number to BCD
;* Input : w <- 8 bit binary number
;* Output: AARGB0:AARGB1		; (AARG16) converted number (3 digit right justified)
;* Clock : ?	
;**************************************************************************
	ifdef	INC_BIN2BCD8
bin2bcd8	clrf	AARGB1
		clrf	AARGB0
				
BCD_b2b_1	addlw	-100
		jnc	BCD_b2b_2
		incf	AARGB0,f
		goto	BCD_b2b_1
		
BCD_b2b_2	addlw	100

BCD_b2b_3	addlw	-10
		jnc	BCD_b2b_4
		incf	AARGB1,f
		goto	BCD_b2b_3
		
BCD_b2b_4	addlw	10
		swapf	AARGB1,f
		addwf	AARGB1,f
		return
	endif



;***** BCD_bin2bcd16i *****************************************************
;* Convert a 16 bit binary number to BCD
;* Input : w <-address of 16 bit binary number (Bank must be selected!)
;* Output: AARGB0:AARGB1:AARGB2	; (AARG24) converted number (5 digit)
;* Clock : ?	
;**************************************************************************
	ifdef	INC_BIN2BCD16I
bin2bcd16i	movwf	FSR
		
		movf	INDF,w
		movwf	BARGB0
		incf	FSR,f
		movf	INDF,w
		movwf	BARGB1
						; cont. with bin2bcd16
	endif


;***** BCD_bin2bcd16 ******************************************************
;* Convert a 16 bit binary number to BCD
;* Input : BARG16 <-16 bit binary number
;* Output: AARGB0:AARGB1:AARGB2	; (AARG24) converted number (5 digit)
;* Clock : ?	
;**************************************************************************
	ifdef	INC_BIN2BCD16
bin2bcd16	movlw	16
		movwf   LOOPCOUNT
		clrf	AARGB0 
		clrf	AARGB1 
		clrf	AARGB2 
		goto	_startb16

_adjdec16	movlw	0x33 
		addwf	AARGB0,f 
		addwf	AARGB1,f 
		addwf	AARGB2,f 

		movlw	0x03 
		btfss	AARGB0,3 
		 subwf	AARGB0,f 
		btfss	AARGB1,3 
		 subwf	AARGB1,f 
		btfss	AARGB2,3 
		 subwf	AARGB2,f 

		movlw	0x30 
		btfss	AARGB0,7 
		subwf	AARGB0,f 
		btfss	AARGB1,7 
		subwf	AARGB1,f 
		btfss	AARGB2,7 
		subwf	AARGB2,f 

_startb16	rlf	BARGB1,f 
		rlf	BARGB0,f 
		rlf	AARGB2,f 
		rlf	AARGB1,f 
		rlf	AARGB0,f 
		decfsz	LOOPCOUNT,f 
		goto	_adjdec16
		return 
	endif



;***** bcd_dec8 ***********************************************************
;* Decrement a BCD number
;* Input : BARG8 <- 8 bit BCD-number	;
;* Output: -				; Z flag set if equal to 99 (underflow)
;* Clock : 13
;**************************************************************************
	ifdef	INC_BCD_DEC8
bcd_dec8	movlw	BARG8
					; continue with BCD_dec8i
	endif



;***** bcd_dec8i **********************************************************
;* Decrement a BCD number
;* Input : w <- address of BCD-number	; Bank must be selected!
;* Output: -				; Z flag set if equal to 99 (underflow)
;* Clock : 12		
;**************************************************************************
	ifdef	INC_BCD_DEC8I
bcd_dec8i	movwf	FSR

		movlw	1
		subwf	INDF, f		; Sub 1 to set C and DC flags	
		movlw	06h		; Set up for one's underflow
		skipc			; If C set, possible one's underflow
		movlw	66h		; Set up for ten's underflow
		skipdc			; Leave alone if no underflow
		subwf	INDF, f		; Else, sub either 06h or 66h depending
					; on a one's or ten's underflow
		movlw	99h
		subwf	INDF, w		; Z flag set if equal to 99 (underflow)

		return
	endif



;***** BCD_dec8 ***********************************************************
;* Increment a BCD number
;* Input : BARG8 <- 8 bit BCD-number	;
;* Output: -				; Z will be set if overflow to zero
;* Clock : 13
;**************************************************************************
	ifdef	INC_BCD_INC8
bcd_inc8	movlw	BARG8
					; continue with BCD_inc8i
	endif



;***** BCD_inc8i **********************************************************
;* Increment a BCD number
;* Input : w <- address of BCD-number	; Bank must be selected!
;* Output: -  				; Z will be set if overflow to zero
;* Clock : 12		
;**************************************************************************
	ifdef	INC_BCD_INC8I
bcd_inc8i	movwf	FSR

		movlw	07h		; Test for overflow of one's place
		addwf	INDF,f
		movlw	06h		; Set up for possible sub
		skipdc
		subwf	INDF,f		; If no DC, sub 06h to inc by 1
		
		movlw	60h		; Test for overflow of ten's place
		addwf	INDF,f		; C will be set if INDF = A0h
		skipc			; Leave value alone if overflow
		subwf	INDF,f		; No overflow - restore value

		return			; Z will be set if overflow to zero
	endif



;**************************************************************************

