;**************************************************************************
;* FileName	: CALLDB.INC
;* Date 	: 15.06.2002
;* File version : v1.02
;*
;* Author	: MIKO, Imre
;* Company	:
;* Homepage	: http://majki.fw.hu
;* E-mail	: majki@fw.hu
;*
;*-------------------------------------------------------------------------
;*
;* Description	: Call Database
;*		  ==================================
;*		  CDB_init
;*		  CDB_store
;*		  CDB_delete
;*		  CDB_read
;*		  CDB_getidx
;*		  CDB_setidx
;*		  CDB_read_header
;*		  CDB_write_header
;*		  CDB_read_record
;*		  CDB_write_record
;*
;*
;*-------------------------------------------------------------------------
;* Needed defines in main-modul:
;* 
;*  none
;*
;*-------------------------------------------------------------------------
;* Other possible defines in main-modul:
;*
;*  none
;*
;**************************************************************************
;* Revision History:
;*
;* 1.00	  03/06/2001 	Initial Release
;* 1.01   23/08/2001    - modify the init sequence
;*			- reorganize header-structure
;* 1.02   15/06/2002    - code optimalisation
;*
;**************************************************************************

;***** VARIABLES **********************************************************


;--------------------------------------------------------------------------
CDB_hfirst	EQU	CDB_header	+ CDB_HFIRST
CDB_hlast	EQU	CDB_header	+ CDB_HLAST
CDB_hefirst	EQU	CDB_header	+ CDB_HEFIRST
CDB_helast	EQU	CDB_header	+ CDB_HELAST
CDB_hnofin	EQU	CDB_header	+ CDB_HNOFIN
CDB_hnoflost	EQU	CDB_header	+ CDB_HNOFLOST
CDB_hcdate	EQU	CDB_header	+ CDB_HCDATE	; not used
CDB_hflag	EQU	CDB_header	+ CDB_HFLAG	; not used
CDB_hlvisit	EQU	CDB_header	+ CDB_HLVISIT	; not used
CDB_hdur_in	EQU	CDB_header	+ CDB_HDUR_IN
CDB_hdur_out	EQU	CDB_header	+ CDB_HDUR_OUT

;--------------------------------------------------------------------------
CDB_rnext	EQU	CDB_record	+ CDB_INEXT
CDB_rprev	EQU	CDB_record	+ CDB_IPREV
CDB_rduration	EQU	CDB_record	+ CDB_IDURATION
CDB_rdate	EQU	CDB_record	+ CDB_IDATE
CDB_rnummer	EQU	CDB_record	+ CDB_INUMBER

;--------------------------------------------------------------------------

;***** CDB_init ***********************************************************
;* initialize of database
;* Input : -				; 
;* Output: -				;
;* Clock : ?		
;**************************************************************************
CDB_init	MEM_CLEAR CDB_record, CDB_RECORD_LEN	
		call	CDB_read_header
		bank1
		ERRORLEVEL -302
		incf	CDB_hfirst,w		; if first==0FFh perform a firstinit.
		ERRORLEVEL +302
		bank0
		clrc
		skipnz
		call	CDB_firstinit
		clrf	CDB_current
		return



;***** CDB_firstinit ******************************************************
;* full initialize of database
;* Input : C <- mode			; 0 - full, 1 - preserve times
;* Output: -				;
;* Clock : ?		
;**************************************************************************
CDB_firstinit	movlw	CDB_header	; clear header
		movwf	FSR

		movlw	CDB_HEADER_LEN
		skipnc
		movlw	CDB_HEADERX_LEN
CDB_fi0
		clrf	INDF
		incf	FSR,f
		decw
		jnz	CDB_fi0

;----- create header ------------------------------------------------------
		bank1
		ERRORLEVEL -302			; bank1 !
		movlw	CDB_EEADR_FIRSTREC/CDB_RECORD_LEN
		movwf	CDB_hefirst
		addlw	CDB_NUM_OF_REC-1
		movwf	CDB_helast		

		bank0
		ERRORLEVEL +302
		call	CDB_write_header	; write header into EEPRPOM

			
;----- create empty records chain ------------------------------------------
		movlw	HIGH CDB_EEADR_FIRSTREC
		movwf	EE24_addr_h
		movlw	LOW CDB_EEADR_FIRSTREC
		movwf	EE24_addr_l

		bank1
		ERRORLEVEL -302
		movlw	CDB_EEADR_FIRSTREC/CDB_RECORD_LEN+1
		movwf	CDB_rnext

		movlw	CDB_NUM_OF_REC-1
		movwf	CDB_temp

CDB_fi1		movlw	CDB_record
		movwf	FSR
		movlw	CDB_RECORD_LEN

		bank0
		ERRORLEVEL +302
		EE24_WRITEPAGE

		movlw	CDB_RECORD_LEN		; update EE-address
		addwf	EE24_addr_l,f
		skipnc
		incf	EE24_addr_h,f

		ERRORLEVEL -302
		bank1

		incf	CDB_rnext,f		; update record data

		decfsz	CDB_temp,f
		goto	CDB_fi1

		clrf	CDB_rnext		; write last record
		movlw	CDB_record
		movwf	FSR
		movlw	CDB_RECORD_LEN

		bank0
		ERRORLEVEL +302
		EE24_WRITEPAGE

		return



;***** CDB_store **********************************************************
;* Store a record into call database
;* Input : CDB_record <- record		; 
;* Output: -				;
;* Clock : ?		
;**************************************************************************
CDB_store 	bank1
		ERRORLEVEL -302

		movf	CDB_rduration,w
		xorlw	CDB_STATE_LOST
		skipz
		incf	CDB_hnofin,f
		testw
		skipnz
		incf	CDB_hnoflost,f

		testf	CDB_hefirst	; test empty place
		jnz	CDB_s1

;----- there is not empty place --------------------------------------------

		movf	CDB_hfirst,w	; clear the oldest data
		call	CDB_delete
		bank1
		ERRORLEVEL -302		

;----- there is empty place ------------------------------------------------

CDB_s1		movf	CDB_hlast,w	; temp = last
		movwf	CDB_temp

		movf	CDB_hefirst,w	; last = empty_first
		movwf	CDB_hlast

		testf	CDB_hfirst	; if first == 0 then first = empty_first
		skipnz
		movwf	CDB_hfirst
					; empty_first = empty_first -> next
		CDB_GETIDX CDB_hefirst, CDB_INEXT
		movwf	CDB_hefirst

		testw			; if empty_first == 0 then empty_last = 0
		skipnz
		clrf	CDB_helast

		testf	CDB_temp	; if temp != 0
		jz	CDB_ss1

		movf	CDB_hlast,w	; temp -> next = last
		CDB_SETIDX CDB_temp, CDB_INEXT

CDB_ss1		movf	CDB_temp,w	; prev = temp
		movwf	CDB_rprev

		clrf	CDB_rnext	; next = 0

		movf	CDB_hlast,w
		ERRORLEVEL +302
		bank0

		call	CDB_write_record	; store record
		call	CDB_write_header	; store header
		return


;***** CDB_delete *********************************************************
;* Delete a record from the call database
;* Input : w <- index of record		; 
;* Output: -				;
;* Clock : ?		
;**************************************************************************
CDB_delete	movwf	CDB_temp	; temp = current
		
		bank1
		ERRORLEVEL -302		

		CDB_GETIDX CDB_temp, CDB_IDURATION
		xorlw	CDB_STATE_LOST
		skipz
		decf	CDB_hnofin,f
		testw
		skipnz
		decf	CDB_hnoflost,f

;--------------------------------------------------------------------------
		testf	CDB_helast	; if empty_last != 0 
		jz	CDB_ds1		

		movf	CDB_temp,w	;   - then empty_last -> next = temp
		CDB_SETIDX CDB_helast, CDB_INEXT

;--------------------------------------------------------------------------
CDB_ds1		movf	CDB_temp,w	; empty_last = temp
		movwf	CDB_helast

		testf	CDB_hefirst	; if empty_first == 0 then empty_first = empty_last
		skipnz
		movwf	CDB_hefirst

;--------------------------------------------------------------------------
		movf	CDB_hfirst,w	; if first == last
		xorwf	CDB_hlast,w
		jnz	CDB_dskip4
		
			clrf	CDB_hfirst	
			clrf	CDB_hlast
			clrf	CDB_current
			goto	CDB_dskip3

CDB_dskip4
;--------------------------------------------------------------------------

		movf	CDB_helast,w	; if empty_last == first
		xorwf	CDB_hfirst,w
		jnz	CDB_delse1

					; first = empty_last -> next
			CDB_GETIDX CDB_helast, CDB_INEXT
			movwf	CDB_hfirst
					; current = empty_last -> next
			movwf	CDB_current
					; first -> prev = 0
			clrw
			CDB_SETIDX CDB_hfirst, CDB_IPREV
			goto	CDB_dskip1

CDB_delse1				; temp = empty_last -> prev
			CDB_GETIDX CDB_helast, CDB_IPREV
			movwf	CDB_temp	

					; current = empty_last -> prev
			movwf	CDB_current

					; temp -> next = empty_last -> next
			CDB_GETIDX CDB_helast, CDB_INEXT
			CDB_SETIDX CDB_temp  , CDB_INEXT
CDB_dskip1

;--------------------------------------------------------------------------

		movf	CDB_helast,w	; if empty_last == last
		xorwf	CDB_hlast,w
		jnz	CDB_delse2

					; last = empty_last -> prev
			CDB_GETIDX CDB_helast, CDB_IPREV
			movwf	CDB_hlast
			clrw
			CDB_SETIDX CDB_hlast,  CDB_INEXT
			goto	CDB_dskip2

CDB_delse2
					; temp = empty_last -> next
			CDB_GETIDX CDB_helast, CDB_INEXT
			movwf	CDB_temp
					; temp ->prev = empty_last -> prev
			CDB_GETIDX CDB_helast, CDB_IPREV
			CDB_SETIDX CDB_temp  , CDB_IPREV
CDB_dskip2

;--------------------------------------------------------------------------
CDB_dskip3	clrw			; empty_last -> next = 0
		CDB_SETIDX CDB_helast  , CDB_INEXT

		ERRORLEVEL +302		
		bank0
		call	CDB_write_header
		return



;***** CDB_read ***********************************************************
;* Read a record from the call database
;* Input : w <- mode			; CDB_MODE_FIRST, CDB_MODE_LAST,
;*					; CDB_MODE_NEXT,  CDB_MODE_PREV,
;*					; CDB_MODE_CURR
;* Output: CDB_record <- readed rec.	;
;*	   w <- status			; 0 <- no more rec.  1 <- read success
;* Clock : ?		
;**************************************************************************
CDB_read	bank1
		ERRORLEVEL -302	

		movwf	CDB_temp
		xorlw	CDB_MODE_FIRST
		jnz	CDB_rskip1

			movf	CDB_hfirst,w	; get first
			goto	CDB_read_succ

CDB_rskip1	movf	CDB_temp,w
		xorlw	CDB_MODE_LAST
		jnz	CDB_rskip2

			movf	CDB_hlast,w	; get last
			goto	CDB_read_succ

CDB_rskip2	testf	CDB_current		; current == 0 ?
		jz	CDB_read_fail

		movf	CDB_temp,w
		xorlw	CDB_MODE_PREV
		jnz	CDB_rskip3
		
			CDB_GETIDX CDB_current, CDB_IPREV
			goto	CDB_read_succ

CDB_rskip3	movf	CDB_temp,w
		xorlw	CDB_MODE_NEXT
		jnz	CDB_rskip4
		
			CDB_GETIDX CDB_current, CDB_INEXT
			goto	CDB_read_succ

CDB_rskip4	movf	CDB_temp,w
		xorlw	CDB_MODE_CURR
		jnz	CDB_read_fail

			movf	CDB_current,w

;--------------------------------------------------------------------------
CDB_read_succ	movwf	CDB_current
		testw
		jz	CDB_read_fail	

;--------------------------------------------------------------------------
		bank0
		ERRORLEVEL +302
		movf	CDB_current,w
		call	CDB_read_record
		retlw	1

;--------------------------------------------------------------------------
CDB_read_fail	bank0
		ERRORLEVEL +302	
		retlw	0



;***** CDB_getidx *********************************************************
;* Get indirect index from database
;* Input : CDB_ftemp <- field offset	; 
;*	   w <- index of source record	; swaped!
;* Output: readed index -> w		;
;* Clock : 		
;**************************************************************************
CDB_getidx	bank0
		ERRORLEVEL +302
		movwf	EE24_addr_h
		andlw	0F0h
		addwf	CDB_ftemp,w
		skipnc
		incf	EE24_addr_h,f
		movwf	EE24_addr_l
		movlw	00Fh
		andwf	EE24_addr_h,f
		EE24_READBYTE
		ERRORLEVEL -302
		bank1
		return



;***** CDB_setidx *********************************************************
;* Set indirect index from database
;* Input : CDB_ftemp <- field offset	; 
;*	   w <- index of source record	; swaped!
;*	   CDB_dtemp <- new_index	; new value of the index
;* Output: -				;
;* Clock : 		
;**************************************************************************
CDB_setidx	bank0
		ERRORLEVEL +302
		movwf	EE24_addr_h
		andlw	0F0h
		addwf	CDB_ftemp,w
		skipnc
		incf	EE24_addr_h,f
		movwf	EE24_addr_l
		movlw	00Fh
		andwf	EE24_addr_h,f	
		movf	CDB_dtemp,w
		EE24_WRITEBYTE
		ERRORLEVEL -302
		bank1
		return



;***** CDB_read_header ****************************************************
;* Read database header from EEPROM
;* Input : -				; 
;* Output: header -> CDB_header		;
;* Clock : 		
;**************************************************************************
CDB_read_header	movlw	HIGH CDB_EEADR_HEADER
		movwf	EE24_addr_h
		movlw	LOW CDB_EEADR_HEADER
		movwf	EE24_addr_l
		movlw	CDB_header
		movwf	FSR
		movlw	CDB_HEADER_LEN
		EE24_READPAGE
		return



;***** CDB_write_header ***************************************************
;* Write database header into EEPROM
;* Input : CDB_header			; 
;* Output: -				;
;* Clock : 		
;**************************************************************************
CDB_write_header
		movlw	HIGH CDB_EEADR_HEADER
		movwf	EE24_addr_h
		movlw	LOW CDB_EEADR_HEADER
		movwf	EE24_addr_l
		movlw	CDB_header
		movwf	FSR
		movlw	CDB_HEADER_LEN
		EE24_WRITEPAGE
		return



;***** CDB_read_record ****************************************************
;* Read a record from the call database
;* Input : w <- index of the source record 
;* Output: record -> CDB_record		;
;* Clock : 		
;**************************************************************************
CDB_read_record	movwf	EE24_addr_h
		swapf	EE24_addr_h,f
		movf	EE24_addr_h,w
		andlw	0F0h
		movwf	EE24_addr_l
		movlw	00Fh
		andwf	EE24_addr_h,f
		movlw	CDB_record
		movwf	FSR
		movlw	CDB_RECORD_LEN
		EE24_READPAGE
		return



;***** CDB_write_record ***************************************************
;* Write a record into the call database
;* Input : CDB_record			; record data
;*	   w <- index of the destination record
;* Output: -				;
;* Clock : 		
;**************************************************************************
CDB_write_record
		movwf	EE24_addr_h
		swapf	EE24_addr_h,f
		movf	EE24_addr_h,w
		andlw	0F0h
		movwf	EE24_addr_l
		movlw	00Fh
		andwf	EE24_addr_h,f
		movlw	CDB_record
		movwf	FSR
		movlw	CDB_RECORD_LEN
		EE24_WRITEPAGE
		return


;**************************************************************************
