;**************************************************************************
;* FileName	: CID.ASM
;* Date 	: 26.10.2002
;* File version : v1.03
								list l=0
;* Author	: MIKO, Imre
;* Company	:
;* Homepage	: http://majki.fw.hu
;* E-mail	: majki@fw.hu
;*
;*-------------------------------------------------------------------------
;*	
;* Description	: Caller ID               v1.06
;*		  =============================
;*		  Caller Identifier with MT8843
;*
;*
;*		  Project versions:
;*		  -----------------
;*		  1.00	03/06/2001 	Initial Release
;*
;*		  1.02	23/08/2001	- code optimalisation (size)
;*					- fix delete-bug of call-list
;*					- change numbering order
;*
;*		  1.04  15/06/2002	- code optimalisation (size)
;*					- print number of missed calls
;*					- print call duration during incomming call
;*					- remove second call handling
;*					- fix wrong store
;*					- fix wrong list handling after delete
;*
;*		  1.06  26/10/2002	- reformatting sources
;*					- replace obsolete bcd module with math module
;*					- fix IT bank settings
;*					- fix wrong missed call storage
;*					- start only with OK button
;*
;*
;*-------------------------------------------------------------------------
;*
;* Connections	:	       _______	 _______
;*			      |       \_/	|
;*		     -reset ==| -MCLR	    RB7 |== KEY4_1
;*		        n.c ==| RA0	    RB6 |== KEY4_2
;*		        n.c ==| RA1	    RB5 |== KEY4_3
;*		 NLCD_RESET ==| RA2	    RB4 |== KEY4_4
;*  		   NLCD_CS1 ==| RA3	    RB3 |== NLCD_CS0
;*		        n.c ==| RA4	    RB2 |== NLCD_CLK
;*		        n.c ==| RA5	    RB1 |== NLCD_SI
;*		        GND ==| Vss	    RB0 |== MT8843_TRIGout
;*		    quartz1 ==| OSC1	    Vdd |== +5V
;*		    quartz2 ==| OSC2	    Vss |== GND
;*	          MT8843_CD ==| RC0	    RC7 |== MT8843_HOOKON
;*		  MT8843_DR ==| RC1	    RC6 |== MT8843_PWDN
;*		MT8843_DATA ==| RC2	    RC5 |== MT8843_DCLK
;*		    I2C_SCL ==| RC3	    RC4 |== I2C_SDA
;*			      |_________________|
;*				   PIC16F872 
;*
;*-------------------------------------------------------------------------
;*
;* Resource	: Ports 	  :    18 /   22 pin   ( 82 %)
;*		  Data memory	  :    75 /  128 byte  ( 59 %)
;*		  Prog memory	  :  2021 / 2048 word  ( 98 %)
;*		  EEPROM	  :     0 /   64 byte  (  0 %)
;*		  Timers	  : Timer0
;*		  Interrupt	  : Timer
;*		  CCP		  : -   
;*		  MSSP		  : Master I2C
;*		  AD		  : -
;*
;**************************************************************************
;* Revision History:
;*
;* 1.00	  03/06/2001 	Initial Release
;* 1.01   23/08/2001    New timer
;* 1.02   15/06/2002    Code optimalisation
;* 1.03   26/10/2002    Fix IT bank settings
;*
;**************************************************************************

	list	p=16f872, r=dec, l=0

;***** INCLUDES ***********************************************************

	include "p16f872.inc"
	include	"..\modules\base.inc"

;***** CONFIG ************************************************************

CFG_1	EQU 	_XT_OSC & _WDT_OFF & _PWRTE_ON & _BODEN_ON & _LVP_OFF
CFG_2	EQU	_CPD_OFF & _WRT_ENABLE_OFF & _DEBUG_OFF & _CP_OFF

	__CONFIG CFG_1 & CFG_2
	__IDLOCS 0106h			; version

	EXPAND_MACROS	EXP_COND		; macro expansion mode 

;***** DEFINES ************************************************************
;#define DEBUG				; for some debug message
;#define NO_DELAY			; for fast simulation
;#define NO_RS232			; for fast simulation
;#define NO_EE24
;#define MDEBUG				; for serial debugging

xtal_freq	EQU	3579545	  	; crystal frequency
core_freq	EQU	xtal_freq/4	; core operating frequency

TIMER_TMR0pre	EQU	256	  	; timer0 prescaler value (2, 4, .. 256)
TIMER_TMR0val	EQU	152		; TMR0 init value (1-256)

#define		NLCD_ALPHA


; ----- porta -------------------------------------------------------------

#ifdef MDEBUG
  #define	RS232_TxD	PORTA,0		; out
  #define	RS232_RxD	PORTA,1		; in
#else
  #define	PORTA_0		PORTA,0		; free
  #define	PORTA_1		PORTA,1		; free
#endif

#define		NLCD_RESET	PORTA,2		; out
#define		NLCD_CS1	PORTA,3		; out
#define		PORTA_4		PORTA,4		; free (open drain!)
#define		PORTA_5		PORTA,5		; free

; ----- portb -------------------------------------------------------------

#define		MT8843_TRIGOUT	PORTB,0		; in
#define		NLCD_SI		PORTB,1		; out
#define		NLCD_CLK	PORTB,2		; out
#define		NLCD_CS0	PORTB,3		; out
#define		KEY4_4		PORTB,4		; in
#define		KEY4_3		PORTB,5		; in
#define		KEY4_2		PORTB,6		; in
#define		KEY4_1		PORTB,7		; in

; ----- portc -------------------------------------------------------------

#define		MT8843_CD	PORTC,0		; in
#define		MT8843_DR	PORTC,1		; in
#define		MT8843_DATA	PORTC,2		; in
#define		I2C_SCL		PORTC,3		; out
#define		I2C_SDA		PORTC,4		; in/out
#define		MT8843_DCLK	PORTC,5		; out 
#define		MT8843_PWDN	PORTC,6		; out
#define		MT8843_HOOKON	PORTC,7		; in


;--------------------------------------------------------------------------


#define		MENU_IDLE	CID_idle	; idle function (subrutin)


;***** MACROS *************************************************************

	include "..\modules\wait.mac"
	include	"..\modules\math.mac"
	include	"..\modules\nlcd.mac"
	include "..\modules\timer.mac"
	include	"..\modules\key4.mac"
	include "..\modules\i2c_hw.mac"
	include "..\modules\24cxx.mac"
	include "MT8843.mac"
	include "menu.mac"
	include "callDB.mac"

#ifdef MDEBUG
	include "..\modules\rs232.mac"
#endif

; -------------------------------------------------------------------------


;***** CONSTANT DEFINITIONS ***********************************************

option_val	EQU	00000010b	; Timer source : intclk / 8
adcon0_val	EQU	00000000b	; ADCON0 value (AD off)
adcon1_val	EQU	00000110b	; ADCON1 value (all digital input)
intcon_val	EQU	00100000b	; Interrupt control



;***** VARIABLE DEFINITIONS ***********************************************
	cblock	70h			; common RAM
		w_temp			; variable used for context saving
		status_temp		; variable used for context saving
		fsr_temp		; variable used for context saving

		; ----- CDB module -----
		CDB_temp
		CDB_dtemp
		CDB_ftemp
		CDB_current
	endc

;--------------------------------------------------------------------------
	cblock	0A0h
	  CDB_header:CDB_HEADER_LEN
	  CDB_record:CDB_RECORD_LEN
	endc

;--------------------------------------------------------------------------
	cblock	20h
		KEY4_timer
		visit_counter		; count of the not visited missed call
	endc



;***** ENTRY POINT ********************************************************
		ORG	000h		; processor reset vector
		bank1
		ERRORLEVEL -302
		movlw	option_val
		movwf	OPTION_REG	; set OPTION_REG
		goto	start


;***** IT HANDLER *********************************************************
;* Interrupt handler
;**************************************************************************
		ORG	004h		; interrupt vector location
IT_service	movwf	w_temp		; save off current W register contents
		swapf	STATUS,w	; move STATUS register into W register
		movwf	status_temp	; save off contents of STATUS register

;--------------------------------------------------------------------------
IT_timer	clrf	STATUS		; clear RP0, RP1 bits
		TIMER_UPDATE MT8843_timer
		TIMER_UPDATE MT8843_wtimer
		TIMER_UPDATE KEY4_timer
		TIMER_UPDATE MENU_delay
		TIMER_SETTMR0

;--------------------------------------------------------------------------
IT_restore	swapf	status_temp,w	; retrieve copy of STATUS register
		movwf	STATUS		; restore pre-isr STATUS register contents
		swapf	w_temp,f
		swapf	w_temp,w	; restore pre-isr W register contents
		retfie			; return from interrupt

;--------------------------------------------------------------------------



;***** REAL PROGRAM START *************************************************
start
;----- set configuration --------------------------------------------------
		movlw	adcon1_val	
		movwf	ADCON1		; set ADCON1
		bank0
		ERRORLEVEL +302
		movlw	adcon0_val	
		movwf	ADCON0		; set ADCON0
		movlw	intcon_val	
		movwf	INTCON		; set INTCON

;----- inits --------------------------------------------------------------

#ifdef MDEBUG
		RS232_INIT 9600, 8, 1
#endif
		NLCD_INIT
		KEY4_INIT
		TIMER_INIT
		MT8843_INIT
		EE24_INIT	100 kHz
		CDB_INIT

		TIMER_START KEY4_timer,	20	; 20 mS
		clrf	visit_counter

		goto MENU




;***** CID_idle ***********************************************************
;* Idle function (poll keyboard and check status of phone-line)
;* Input : -             		; 
;* Output: W <- event			; new event
;* Clock : ?  
;**************************************************************************
CID_idle
; ----- test keys. --------------------------------------------------------

CID_idle_skip1	KEY4_GET
		testw
		skipz
		return				; w <- pressed key

; ----- test MITEL chip ---------------------------------------------------

#ifdef MDEBUG
		WAIT 50 Millisec,0
#endif

		call	MT8843_check
		testw
		skipz
		return				; w <- MT event

; ----- test for polling --------------------------------------------------

		TIMER_OUT KEY4_timer, KEY4_timeout
		goto	CID_idle

; ----- poll keys ---------------------------------------------------------

KEY4_timeout	KEY4_POLL
		TIMER_START KEY4_timer,	20	; 20 mS
		goto	CID_idle


;***** INCLUDES ***********************************************************

#ifdef MDEBUG
	include "..\modules\rs232.inc"
#endif

	include "callDB.inc"
	include "menu.inc"
	include "MT8843.inc"
	include "..\modules\24cxx.inc"
	include "..\modules\i2c_hw.inc"
	include	"..\modules\key4.inc"
	include "..\modules\nlcd.inc"
	include	"..\modules\math.inc"
	include "..\modules\wait.inc"

;**************************************************************************
	end				  ; end of program








