;**************************************************************************
;* FileName	: MT8843.INC
;* Date 	: 11.10.2002
;* File version : v1.03
;*
;* Author	: MIKO, Imre
;* Company	:
;* Homepage	: http://majki.fw.hu
;* E-mail	: majki@fw.hu
;*
;*-------------------------------------------------------------------------
;*
;* Description	: MT8843 functions
;*		  ==================================
;*		  MT8843_init
;*		  MT8843_check
;*		  MT8843_inc_duration
;*		  MT8843_readCID
;*		  MT8843_read
;*
;*-------------------------------------------------------------------------
;* Needed defines and includes in main-modul:
;* 
;*  <none>
;*
;*-------------------------------------------------------------------------
;* Other possible defines in main-modul:
;*
;*  <none>
;*
;**************************************************************************
;* Revision History:
;*
;* 1.00	  03/06/2001 	Initial Release
;* 1.01   23/08/2001    - correction of timeing
;* 1.02   15/06/2002    - remove second call handling
;* 1.03   11/10/2002	- replace obsolete bcd module with math module
;*			- fix wrong missed call storage
;*
;**************************************************************************

;***** VARIABLES **********************************************************

	cblock
		MT8843_data		; receive buffer
		MT8843_bits		; loop counter

		MT8843_CID_length	; length of received CID Message
		MT8843_Partlength	; length of part Message
		MT8843_checksum
		MT8843_TEMP
		MT8843_timer
		MT8843_wtimer

		MT8843_phase		; working phase
		MT8843_flags		; flags
		MT8843_calltime_min	; duration of call
		MT8843_calltime_sec	; duration of call

	endc


MT8843_buffer		EQU	CDB_record
MT8843_rduration	EQU	MT8843_buffer	+ CDB_IDURATION
MT8843_rdate		EQU	MT8843_buffer	+ CDB_IDATE
MT8843_rnummer		EQU	MT8843_buffer	+ CDB_INUMBER



;***** MT8843_init ********************************************************
;* initialiation of the MT8843 CID IC
;* Input : -             		; 
;* Output: - 				; 
;* Clock : 16  
;**************************************************************************
MT8843_init
		bsf	MT8843_PWDN
		bank1
		bsf	MT8843_TRIGOUT	; in
		bsf	MT8843_CD	; in
		bsf	MT8843_DR	; in
		bsf	MT8843_DATA	; in
		bcf	MT8843_DCLK	; out 
		bcf	MT8843_PWDN	; out
		bsf	MT8843_HOOKON	; in
		bank0
		bcf	MT8843_DCLK
		bsf	MT8843_PWDN	; Power Down
		clrf	MT8843_phase
		clrf	MT8843_flags
		return



;***** MT8843_check *******************************************************
;* Check state of line / call
;* Input : -             		; 
;* Output: w <- event			; MEVENT_NONE, MEVENT_NEWCALL or
;*					  MEVENT_UPDATE
;* Clock : ?  
;**************************************************************************
MT8843_check	
#ifdef	MDEBUG
		movf	MT8843_phase,w
		RS232_SENDW
#endif
		movlw	HIGH MT8843_ctable
		movwf	PCLATH
		
		movf	MT8843_phase,w
		andlw	00000111b		; range limit
		addlw	LOW MT8843_ctable
		skipnc
		incf	PCLATH,f
		movwf	PCL

MT8843_ctable	goto	MT8843_pidle
		goto	MT8843_p1string
		goto	MT8843_pcdok
		goto	MT8843_pcidok
		goto	MT8843_phookon
		goto	MT8843_pidle		; It was MT8843_pnextring
		goto	MT8843_poutgoing
		goto	MT8843_pidle		; reserved


;**************************************************************************
;* Phase idle
;**************************************************************************
MT8843_pidle	clrf	MT8843_flags
		clrf	MT8843_calltime_sec
		clrf	MT8843_calltime_min

		btfss	MT8843_TRIGOUT
		goto	MT8843_pidle_e1		; event 1 - ring
		btfss	MT8843_HOOKON
		goto	MT8843_pidle_e2		; event 2 - hooked on
		retlw	MEVENT_NONE


; ----- event ring --------------------------------------------------------
MT8843_pidle_e1	TIMER_WAIT MT8843_wtimer, 200	; 200 mS
		btfsc	MT8843_TRIGOUT
		retlw	MEVENT_NONE		; It was a false ring trigger.

		bcf	MT8843_PWDN		; Power up MT8843
		TIMER_START MT8843_timer, 3000	; 3 Seconds
		movlw	MT8843_PHASE_1STRING	; Phase = MT8843_PHASE_1STRING
		movwf	MT8843_phase
		bsf	MT8843_flag_incall
		retlw	MEVENT_NONE

; ----- event hooked on ---------------------------------------------------
MT8843_pidle_e2	bsf	MT8843_flag_outhook	
		TIMER_START MT8843_timer, 11000	; 11 Seconds
		movlw	MT8843_PHASE_OUTGOING	; Phase = MT8843_PHASE_OUTGOING
		movwf	MT8843_phase
		clrf	visit_counter		; clear visit counter
		retlw	MEVENT_NEWCALL



;**************************************************************************
;* Phase first ring
;**************************************************************************
MT8843_p1string	TIMER_OUT MT8843_timer, MT8843_cd_timeout
		btfss	MT8843_HOOKON
		goto	MT8843_p1st_hon		; hooked on
		btfsc	MT8843_CD
		retlw	MEVENT_NONE

; ----- Carrier detect active ---------------------------------------------
		movlw	MT8843_PHASE_CDOK	; Phase = MT8843_PHASE_CDOK
		movwf	MT8843_phase
		retlw	MEVENT_NONE

; ----- CID was not delivered ---------------------------------------------
MT8843_cd_timeout				; CID was not delivered ..
MT8843_p1st_hon					; .. or hooked on
		bsf	MT8843_PWDN		; Power down MT8843
		movlw	MT8843_PHASE_CIDOK	; Phase = MT8843_PHASE_CIDOK
		movwf	MT8843_phase
		retlw	MEVENT_NEWCALL		; display XXX



;**************************************************************************
;* Phase Carrier detect ok
;**************************************************************************
MT8843_pcdok	call	MT8843_readCID
		xorlw	MT8843_retOK
		jnz	MT8843_pcdok_nok

; ----- Read of CID was successful ----------------------------------------
		bsf	MT8843_flag_readcidok
MT8843_pcdok_nok
		movlw	MT8843_PHASE_CIDOK	; Phase = MT8843_PHASE_CIDOK
		movwf	MT8843_phase
		TIMER_START MT8843_timer, 6000	; 6 Seconds
		retlw	MEVENT_NEWCALL		; display CID or XXX


;**************************************************************************
;* Phase CID ok
;**************************************************************************
MT8843_pcidok	btfss	MT8843_HOOKON
		goto	MT8843_cok_hon		; hooked on
		btfss	MT8843_TRIGOUT
		goto	MT8843_cok_ring		; ring
		TIMER_OUT MT8843_timer, MT8843_cok_timeout
		retlw	MEVENT_NONE

; ----- hooked on ---------------------------------------------------------
MT8843_cok_hon	TIMER_WAIT MT8843_wtimer, 200	; 200 mS
		btfsc	MT8843_HOOKON
		retlw	MEVENT_NONE		; It was a false hookon trigger.

		bsf	MT8843_flag_inhook
		TIMER_START MT8843_timer, 1000	; 1000 mS
		movlw	MT8843_PHASE_HOOKON	; Phase = MT8843_PHASE_HOOKON
		movwf	MT8843_phase
		retlw	MEVENT_UPDATE

; ----- new ring ----------------------------------------------------------
MT8843_cok_ring	TIMER_WAIT MT8843_wtimer, 200	; 200 mS
		btfsc	MT8843_TRIGOUT
		retlw	MEVENT_NONE		; It was a false ring trigger.

		TIMER_START MT8843_timer, 5500	; 5.5 Seconds
		; movlw	MT8843_PHASE_CIDOK	; Phase = MT8843_PHASE_CIDOK
		; movwf	MT8843_phase
		retlw	MEVENT_NONE

; ----- Missed call -------------------------------------------------------
MT8843_cok_timeout
		btfsc	MT8843_flag_stored	; if stored as incomming, do not
		goto	MT8843_cok_nostore	; store as missed. (workaround)

		bank1				; store as a lost call
		ERRORLEVEL -302
		movlw	CDB_STATE_LOST
		movwf	CDB_rduration
		bank0
		ERRORLEVEL -302
		CDB_STORE
	
MT8843_cok_nostore
		movlw	MT8843_PHASE_IDLE	; Phase = MT8843_PHASE_IDLE
		movwf	MT8843_phase
		bcf	MT8843_flag_incall
		BCD_INC8I visit_counter		; inc. missed call visit counter
		retlw	MEVENT_UPDATE


;**************************************************************************
;* Phase hook on
;**************************************************************************
MT8843_phookon	TIMER_OUT MT8843_timer, MT8843_in_update
		btfss	MT8843_HOOKON
		retlw	MEVENT_NONE

		TIMER_WAIT MT8843_wtimer, 200	; 200 mS
		btfss	MT8843_HOOKON
		retlw	MEVENT_NONE		; It was a false hookoff trigger.		
		
; ----- event hooked off --------------------------------------------------
		btfss	MT8843_flag_readcidok
		goto	MT8843_skipstr

		movlw	CDB_rduration		; store call 
		movwf	FSR
		movf	MT8843_calltime_min,w
		movwf	INDF
		incf	FSR,f
		movf	MT8843_calltime_sec,w
		movwf	INDF
		CDB_STORE
		bsf	MT8843_flag_stored	; set stored flag

MT8843_skipstr	movlw	MT8843_PHASE_IDLE	; Phase = MT8843_PHASE_IDLE
		movwf	MT8843_phase
		bcf	MT8843_flag_inhook
		bcf	MT8843_flag_incall
		retlw	MEVENT_UPDATE
			
; ----- increment callduration --------------------------------------------
MT8843_in_update
		call	MT8843_inc_duration
		TIMER_START MT8843_timer, 1000	; 1 Seconds
		retlw	MEVENT_UPDATE


;**************************************************************************
;* Phase outgoing call
;**************************************************************************
MT8843_poutgoing
		TIMER_OUT MT8843_timer, MT8843_out_update
		btfss	MT8843_HOOKON
		retlw	MEVENT_NONE
		
; ----- event hooked off --------------------------------------------------
		call	CDB_write_header	; store callduration

		movlw	MT8843_PHASE_IDLE	; Phase = MT8843_PHASE_IDLE
		movwf	MT8843_phase
		bcf	MT8843_flag_outhook	
		TIMER_WAIT MT8843_wtimer, 500	; hookoff delay
		retlw	MEVENT_UPDATE

; ----- increment callduration --------------------------------------------
MT8843_out_update
		call	MT8843_inc_duration
		TIMER_START MT8843_timer, 1000	; 1 Seconds
		retlw	MEVENT_UPDATE


;***** MT8843_inc_duration ************************************************
;* Increment call duration (called in every second during the call)
;* Input : -             		; 
;* Output: - 				; 
;* Clock : ?
;**************************************************************************
MT8843_inc_duration
		BCD_INC8I MT8843_calltime_sec	; increment current call duration
		movf	MT8843_calltime_sec,w
		xorlw	60h			; 60th seconds ?
		jnz	MT8843_inc_skip		

		clrf	MT8843_calltime_sec
		BCD_INC8I MT8843_calltime_min

MT8843_inc_skip	movlw	CDB_hdur_out+2		; increment summa call duration
		btfss	MT8843_flag_outhook
		movlw	CDB_hdur_in+2
		movwf	MT8843_TEMP
		
		call	bcd_inc8i
		movf	INDF,w
		xorlw	60h			; 60th seconds ?
		skipz
		return

		clrf	INDF
		decf	MT8843_TEMP,f
		movf	MT8843_TEMP,w
		call	bcd_inc8i
		movf	INDF,w
		xorlw	60h			; 60th minutes ?
		skipz
		return

		clrf	INDF
		decf	MT8843_TEMP,w
		call	bcd_inc8i
		return




;***** MT8843_readCID *****************************************************
;* Read and process Caller ID Information form the MT8843 IC
;* Input : -             		; 
;* Output: w <- result			; MT8843_retOK or MT8843_retNOK
;* Clock : ?  
;**************************************************************************
MT8843_readCID	
; ----- Start timer -------------------------------------------------------

		TIMER_START MT8843_timer, 4000	; 4 Seconds

; ----- wait for 2 bytes of channel seizure (55h) -------------------------

MT8843_waitCS	call	MT8843_read
		jz	MT8843_read_timeout
		xorlw	55h
		jnz	MT8843_waitCS

		call	MT8843_read
		jz	MT8843_read_timeout
		xorlw	55h
		jnz	MT8843_waitCS

; ----- Wait for message type word ----------------------------------------

MT8843_waitMDMF	call	MT8843_read
		jz	MT8843_read_timeout

		clrf	MT8843_checksum
		addwf	MT8843_checksum,f

		sublw	80h				; MDMF ?
		jnz	MT8843_waitMDMF


; ----- Read message length -----------------------------------------------

		call	MT8843_read
		jz	MT8843_read_timeout
		movwf	MT8843_CID_length
		addwf	MT8843_checksum,f

; ----- Read message words and compute checksum ---------------------------

		MEM_CLEAR MT8843_buffer, CDB_RECORD_LEN	; clear buffer
		
MT8843_readnext	call	MT8843_read			; read part type
		jz	MT8843_read_timeout
		decf	MT8843_CID_length,f
		addwf	MT8843_checksum,f
		movwf	MT8843_TEMP
		
		call	MT8843_read			; read part length
		jz	MT8843_read_timeout
		decf	MT8843_CID_length,f
		addwf	MT8843_checksum,f
		movwf	MT8843_Partlength

		subwf	MT8843_CID_length,w		; check part length
		jnc	MT8843_read_failed
	
		movf	MT8843_TEMP,w
		sublw	01h
		jz	MT8843_parttime

		movf	MT8843_TEMP,w
		sublw	02h
		jz	MT8843_partCID

		movf	MT8843_TEMP,w
		sublw	04h
		jz	MT8843_partnoCID
		
; ----- Skip part ---------------------------------------------------------
MT8843_skipread call	MT8843_read
		jz	MT8843_read_timeout
		addwf	MT8843_checksum,f
		decf	MT8843_CID_length,f
		decfsz	MT8843_Partlength,f
		goto	MT8843_skipread
		
		goto	MT8843_readend

; ----- Read time ---------------------------------------------------------
MT8843_parttime	movlw	MT8843_rdate-1
		movwf	FSR
		goto	MT8843_part_loop


; ----- Read Caller number ------------------------------------------------
MT8843_partCID  movf	MT8843_Partlength,w	; check CID length
		sublw	CDB_CNUM_LEN*2 -1
		jnc	MT8843_read_failed	

		movlw	MT8843_rnummer
		movwf	FSR
		
		swapf	MT8843_Partlength,w	; store length
		andlw	0F0h
		movwf	INDF

		call	MT8843_read		; store 1. num.
		jz	MT8843_read_timeout
		addwf	MT8843_checksum,f
		decf	MT8843_CID_length,f
		andlw	00Fh
		iorwf	INDF,f
		decf	MT8843_Partlength,f


; ----- Read and pack date or number --------------------------------------
MT8843_part_loop
		incf	FSR,f

		call	MT8843_read		; store n. num.
		jz	MT8843_read_timeout
		addwf	MT8843_checksum,f
		decf	MT8843_CID_length,f
		movwf	MT8843_TEMP
		swapf	MT8843_TEMP,w
		andlw	0F0h
		movwf	INDF
		
		decf	MT8843_Partlength,f
		jz	MT8843_readend

		call	MT8843_read		; store n+1. num.
		jz	MT8843_read_timeout
		addwf	MT8843_checksum,f
		decf	MT8843_CID_length,f
		andlw	00Fh
		iorwf	INDF,f
		
		decfsz	MT8843_Partlength,f
		goto	MT8843_part_loop
		
		goto	MT8843_readend


; ----- Read no CID Information -------------------------------------------
MT8843_partnoCID
		call	MT8843_read
		jz	MT8843_read_timeout
		addwf	MT8843_checksum,f
		decf	MT8843_CID_length,f

; -------------------------------------------------------------------------
MT8843_readend	testf	MT8843_CID_length
		jnz	MT8843_readnext

; ----- Read and verify checksum ------------------------------------------

		call	MT8843_read
		jz	MT8843_read_timeout

		bsf	MT8843_PWDN			; MT8843 power off

		addwf	MT8843_checksum,f
		jnz	MT8843_CRCfailed		

		retlw	MT8843_retOK

; ----- Read of CID failed ------------------------------------------------
MT8843_read_failed
MT8843_read_timeout
MT8843_CRCfailed
		bsf	MT8843_PWDN			; MT8843 power off
		retlw	MT8843_retNOK



;***** MT8843_read ********************************************************
;* Read one byte from the MT8843 IC (serial communication)
;* Input : -              		; 
;* Output: w <- readed byte		; 
;*	   Z <- 1 if timeout
;* Clock : ?   
;**************************************************************************
MT8843_read
MT8843_wdr1	btfss	MT8843_DR	; wait DR
		goto	MT8843_wdr2
		TIMER_OUT MT8843_timer, MT8843_read_to
		goto	MT8843_wdr1
MT8843_wdr2
		WAIT	5 Microsec, 0

		movlw	8		; 8 bit
		movwf	MT8843_bits	

		clrf	MT8843_data	; clear receive buffer

MT8843_rloop	bsf	MT8843_DCLK
	
		WAIT	5 Microsec, 0

		bcf	MT8843_DCLK

		bcf	STATUS,C
		rrf	MT8843_data,f
		btfsc	MT8843_DATA
		bsf	MT8843_data,7
		
		WAIT	5 Microsec, 0

		decfsz	MT8843_bits,f	
		goto	MT8843_rloop

		movf	MT8843_data,w

		bcf	STATUS,Z
		return

MT8843_read_to	bsf	STATUS,Z
		return

;**************************************************************************


